"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

(function () {
  var enterModule = typeof reactHotLoaderGlobal !== 'undefined' ? reactHotLoaderGlobal.enterModule : undefined;
  enterModule && enterModule(module);
})();

var __signature__ = typeof reactHotLoaderGlobal !== 'undefined' ? reactHotLoaderGlobal.default.signature : function (a) {
  return a;
};

const _default = function (_ref, idAccessor) {
  let {
    nodes,
    links
  } = _ref;
  let {
    nodeFilter = () => true,
    onLoopError = loopIds => {
      throw `Invalid DAG structure! Found cycle in node path: ${loopIds.join(' -> ')}.`;
    }
  } = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  // linked graph
  const graph = {};
  nodes.forEach(node => graph[idAccessor(node)] = {
    data: node,
    out: [],
    depth: -1,
    skip: !nodeFilter(node)
  });
  links.forEach(_ref2 => {
    let {
      source,
      target
    } = _ref2;
    const sourceId = getNodeId(source);
    const targetId = getNodeId(target);
    if (!graph.hasOwnProperty(sourceId)) throw `Missing source node with id: ${sourceId}`;
    if (!graph.hasOwnProperty(targetId)) throw `Missing target node with id: ${targetId}`;
    const sourceNode = graph[sourceId];
    const targetNode = graph[targetId];
    sourceNode.out.push(targetNode);

    function getNodeId(node) {
      return typeof node === 'object' ? idAccessor(node) : node;
    }
  });
  const foundLoops = [];
  traverse(Object.values(graph));
  const nodeDepths = Object.assign({}, ...Object.entries(graph).filter(_ref3 => {
    let [, node] = _ref3;
    return !node.skip;
  }).map(_ref4 => {
    let [id, node] = _ref4;
    return {
      [id]: node.depth
    };
  }));
  return nodeDepths;

  function traverse(nodes) {
    let nodeStack = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    let currentDepth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

    for (let i = 0, l = nodes.length; i < l; i++) {
      const node = nodes[i];

      if (nodeStack.indexOf(node) !== -1) {
        const loop = [...nodeStack.slice(nodeStack.indexOf(node)), node].map(d => idAccessor(d.data));

        if (!foundLoops.some(foundLoop => foundLoop.length === loop.length && foundLoop.every((id, idx) => id === loop[idx]))) {
          foundLoops.push(loop);
          onLoopError(loop);
        }

        continue;
      }

      if (currentDepth > node.depth) {
        // Don't unnecessarily revisit chunks of the graph
        node.depth = currentDepth;
        traverse(node.out, [...nodeStack, node], currentDepth + (node.skip ? 0 : 1));
      }
    }
  }
};

var _default2 = _default;
exports.default = _default2;
;

(function () {
  var reactHotLoader = typeof reactHotLoaderGlobal !== 'undefined' ? reactHotLoaderGlobal.default : undefined;

  if (!reactHotLoader) {
    return;
  }

  reactHotLoader.register(_default, "default", "/Users/mizy/projects/nebula-explorer-ent/modules/ForceGraph/src/dagDepths.js");
})();

;

(function () {
  var leaveModule = typeof reactHotLoaderGlobal !== 'undefined' ? reactHotLoaderGlobal.leaveModule : undefined;
  leaveModule && leaveModule(module);
})();